@@ -8,18 +8,47 @@
  */
 
 
-#include<sys/user_config.h>
-#include<types.h>
-#include<lib.h>
-#include<lock.h>
-#include<device.h>
-#include<mm.h>
-#include<proc.h>
-#include<errno.h>
-#include<net/net.h>
-#include<net/ether.h>
-#include<net/ip.h>
-#include<net/arp.h>
+#include <sys/config.h>
+#include <sys/param.h>
+#include <sys/types.h>
+#include <kern/lib.h>
+#include <kern/lock.h>
+#include <kern/device.h>
+#include <kern/mm.h>
+#include <kern/proc.h>
+#include <kern/errno.h>
+#include <kern/list.h>
+#include <net/net.h>
+#include <net/ethernet.h>
+#include <net/ip.h>
+#include <net/arp.h>
+
+
+//#define DEBUG_ARP 1
+#ifdef DEBUG_ARP
+	#define STATIC
+	#define INLINE
+#else
+	#define STATIC static
+	#define INLINE	inline
+#endif
+
+
+/**************************************************************************
+ *
+ * ARPå
+ *
+ **************************************************************************/
+
+
+typedef struct{
+	uint ip;
+	char mac[6];
+	char next;
+}ARP_CACHE;
+
+
+//================================== PRIVATE ============================================
 
 
 enum{
@@ -34,32 +63,12 @@ enum{
 	OPE_CODE_ARP_REP= 2,	/* ARP repry. */
 	OPE_CODE_RARP_REQ=3,	/* RARP request. */
 	OPE_CODE_RARP_REP=4,	/* RARP repry. */
+
+	ARP_CACHE_NUM = 64,					/* ARPå */
+	ARP_CACHE_HASH = ARP_CACHE_NUM / 2,	/* ARPåϥå */
 };
 
 
-/**************************************************************************
- *
- * ARPå
- *
- **************************************************************************/
-
-
-enum{
-	ARP_CACHE_NUM=64,				/* ARPå */
-	ARP_CACHE_HASH=ARP_CACHE_NUM/2,	/* ARPåϥå */
-};
-
-
-typedef struct{
-	uint ip;
-	char mac[6];
-	char next;
-}ARP_CACHE;
-
-
-//================================== PRIVATE ============================================
-
-
 static ARP_CACHE arpCache[ARP_CACHE_NUM];
 static char cacheEmpty=0;
 static char hashTbl[ARP_CACHE_HASH];
@@ -118,10 +127,12 @@ static inline char *searchCache(uint ip)
 	uchar tbl;
 	int i;
 
-
-	tbl=(ip>>24)%ARP_CACHE_HASH;
-	for(i=hashTbl[tbl];i!=-1;i=arpCache[i].next)
-		if(arpCache[i].ip==ip)return arpCache[i].mac;
+	tbl = (ip >> 24) % ARP_CACHE_HASH;
+	for (i = hashTbl[tbl]; i != -1; i = arpCache[i].next){
+		if (arpCache[i].ip == ip){
+			return arpCache[i].mac;
+		}
+	}
 
 	return NULL;
 }
@@ -203,8 +214,8 @@ static void transArp(int num, uint dstip, char *dstmac, ushort opecode)
 	head.hardAddrLen = 6;
 	head.protAddrLen = 4;
 	head.opeCode = swapWord(opecode);
-	memcpy(head.srcMac, ethDev[num].mac, 6);
-	head.srcIp = ethDev[num].ip;
+	getEtherMac(num, head.srcMac);
+	head.srcIp = getEtherIp(num);
 	if (dstmac == broadcastMac){
 		memset(head.dstMac,0,6);
 	}
@@ -218,9 +229,9 @@ static void transArp(int num, uint dstip, char *dstmac, ushort opecode)
 	tbi.size[0] = sizeof(ARP_HEADER);
 	tbi.size[1] = 0;
 	tbi.size[2] = 0;
-	tbi.type = DIX_TYPE_ARP;
+	tbi.type = ETHERTYPE_ARP;
 
-	startEmu(ethDev[num].ifnet, &tbi, dstmac);
+	transEther(getEtherIfnet(num), &tbi, dstmac);
 }
 
 
@@ -230,117 +241,152 @@ static void transArp(int num, uint dstip, char *dstmac, ushort opecode)
  *
  **************************************************************************/
 
-
-typedef struct REPLY_WAIT{
-	struct REPLY_WAIT *next;
-	uint               ip;			/* Request IP address. */
-	int volatile       repFlg;		/* Reply flag. */
-	WAIT_INTR          wait;
-	char               mac[6];		/* Reply mac buffer. */
-}REPLY_WAIT;
-
-
 //================================== PRIVATE ============================================
 
+typedef struct REPLY_WAIT{
+	OBJ_LIST	list;		// 󥯥ꥹ
+	uint32_t	ip;			/* Request IP address. */
+	struct proc	*proc;
+	int			isReply;	/* Reply flag. */
+	char		mac[6];		/* Reply mac buffer. */
+}REPLY_WAIT;
 
-static REPLY_WAIT *replyWait;
-
-
-//================================== PROTECTED ==========================================
+enum{
+	RETRY_COUNT		= 3,	// ȥ饤
+	REQUEST_TIMEOUT	= 2,	// ꥯȥॢ
+};
 
+static OBJ_LIST *replyWaitHead;		// Ԥ󥯥إå
+static int waitGate;				// Ԥѥå
 
 /*
- * Receive reply.
- * parameters : source IP,source MAC
- * return : 0 or task switch=1
+ * ARPν
  */
-static int receiveReply(uint srcip,char *mac)
+STATIC void readyReply(
+	const uint32_t dstIp,
+	REPLY_WAIT *m_replyWait)
 {
-	REPLY_WAIT *p;
+	int eflags;
 
+	// 
+	m_replyWait->ip = dstIp;
+	m_replyWait->proc = get_current_task();
+	m_replyWait->isReply = NO;
 
-	/* replyWaitθ */
-	if ((p = replyWait) == NULL){
-		return 0;
+	// 󥯤³
+	eflags = enterCli();
+	enter_spinlock(&waitGate);
+	{
+		addHeadPrevList(&replyWaitHead, &m_replyWait->list);
 	}
-	while (p->ip != srcip){
-		if ((p = p->next) == NULL){
-			return 0;
+	exit_spinlock(&waitGate);
+	exitCli(eflags);
+}
+
+
+// return : YES or NO
+STATIC INLINE int isReply(REPLY_WAIT *wait)
+{
+	return wait->isReply;
+}
+
+
+STATIC INLINE void releaseReplyWait(REPLY_WAIT *wait)
+{
+	// 󥯤
+	int eflags = enterCli();
+	enter_spinlock(&waitGate);
+	{
+		removeFromHeadList(&replyWaitHead, &wait->list);
+	}
+	exit_spinlock(&waitGate);
+	exitCli(eflags);
+}
+
+
+//Գ
+// Receive reply.
+// return : 0 or task switch = 1
+static int receiveReply(const uint32_t srcIp, char *mac)
+{
+	int taskSwitch = 0;
+
+	if (replyWaitHead == NULL) {
+		return taskSwitch;
+	}
+
+	// Ԥ򸡺
+	enter_spinlock(&waitGate);
+	{
+		REPLY_WAIT *waitHead = getHeadObj(OFFSETOF(REPLY_WAIT, list), replyWaitHead);
+		REPLY_WAIT *wait = waitHead;
+
+		if (wait != NULL) {
+			do {
+				if (wait->ip == srcIp){
+					memcpy(wait->mac, mac, 6);
+					wait->isReply = YES;
+					sigWakeSoon(wait->proc);
+					taskSwitch = 1;
+					break;
+				}
+
+				wait = getNextList(OFFSETOF(REPLY_WAIT, list), &wait->list);
+			} while (wait != waitHead);
 		}
 	}
-	memcpy(p->mac, mac, 6);
-	p->repFlg = 1;
+	exit_spinlock(&waitGate);
 
-	return wake_intr(&p->wait,TASK_SIGNAL_WAIT);		/* Ԥץ򵯤 */
+	return taskSwitch;
 }
 
 
 //================================== PUBLIC =============================================
 
 
-/*
- * Get destination MAC.
- * parameters : device number,destination ip address,mac address buffer
- * return : 0 or error number
- */
-int getMac(int num,uint ip,char *mac)
+// Get destination MAC.
+// return : error number
+int getMac(const int num, const uint32_t dstIp, char *m_mac)
 {
-	enum{
-		RETRY_COUNT = 3,		/* ȥ饤 */
-		REQUEST_TIMEOUT = 2000,	/* ꥯȥॢȥߥá */
-	};
-	char *retmac;
-	int rest;
+	char *cacheMac;
 	REPLY_WAIT *wait;
-	REPLY_WAIT *p;
+	int error;
 	int i;
 
-	/* å򸡺 */
-	retmac = searchCache(ip);
-	if (retmac != NULL){
-		memcpy(mac, retmac, 6);
-		return 0;
+	// å򸡺
+	cacheMac = searchCache(dstIp);
+	if (cacheMac != NULL){
+		memcpy(m_mac, cacheMac, 6);
+		return NOERR;
 	}
 
-	/*  */
-	wait = kmalloc(sizeof(REPLY_WAIT));
-	if (wait == NULL){
+	// 
+	wait = kmalloc(sizeof(*wait));
+	if (wait == NULL) {
 		return -ENOMEM;
 	}
-	memset(wait, 0, sizeof(*wait));
-	wait->ip = ip;
-	wait->next = replyWait;
-	replyWait = wait;
+	readyReply(dstIp, wait);
 
-	/* ꥯARP */
-	for (i = 0;;){
-		transArp(num, ip, broadcastMac, OPE_CODE_ARP_REQ);
-		wait_intr(&wait->wait, REQUEST_TIMEOUT, TASK_SIGNAL_WAIT);
-		if (wait->repFlg){
-			addCache(ip, wait->mac);
-			memcpy(mac, wait->mac, 6);
-			rest = 0;
-			break;
-		}
-		if (RETRY_COUNT <= ++i){
-			rest = -ENETUNREACH;
+	// ꥯARP
+	error = -ENETUNREACH;
+	for (i = 0; i < RETRY_COUNT; ++i){
+		transArp(num, dstIp, broadcastMac, OPE_CODE_ARP_REQ);
+
+		sys_sleep(REQUEST_TIMEOUT);
+
+		if (isReply(wait) == YES){
+			addCache(dstIp, wait->mac);
+			memcpy(m_mac, wait->mac, 6);
+			error = NOERR;
 			break;
 		}
 	}
 
-	/*  */
-	if (replyWait == wait){
-		replyWait = wait->next;
-	}
-	else{
-		for (p = replyWait; p->next != wait; p = p->next){;
-		}
-		p->next = wait->next;
-	}
-
+	// 
+	releaseReplyWait(wait);
 	kfree(wait);
 
-	return rest;
+	return error;
 }
 
 
@@ -354,7 +400,7 @@ int getMac(int num,uint ip,char *mac)
 //================================== PUBLIC =============================================
 
 
-/*
+/*Գ
  * Receive ARP frame.
  * parameters : device number,ARP_HEADER address
  * return : task switch on=1,off=0
@@ -378,7 +424,7 @@ printk("ARP_HEADER htype=%x,ptype=%x,hsize=%x,psize=%x,opecode=%x\n"
        dstip[0],dstip[1],dstip[2],dstip[3]);
 ***************************************************************************************************************/
 	/* IPγǧ */
-	if (head->dstIp != ethDev[num].ip){
+	if (head->dstIp != getEtherIp(num)){
 		return 0;
 	}
 
@@ -426,7 +472,7 @@ int test_arp()
 	char mac[6];
 
 	printk("test_arp\n");
-	if(getMac(0,IP_ADDR(172,25,0,2),mac)<0)
+	if(getMac(0,IP_ADDR(172,25,0,2),mac) != NOERR)
 		printk("Failed getMac!\n");
 	else
 		printk("MAC=%d-%d-%d-%d-%d-%d\n",mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
