@@ -10,7 +10,8 @@
 #define icmp_h
 
 
-#include<types.h>
+#include<sys/types.h>
+#include<sys/mbuf.h>
 #include<net/net.h>
 #include<net/netlib.h>
 #include<sys/icmp.h>
@@ -19,7 +20,7 @@
 #ifndef ASM_FILE
 
 
-extern int receiveIcmp(IP_HEADER*);
+extern int receiveIcmp(struct mbuf *);
 extern void releaseSockIcmp(SOCKET*);
 extern int initIcmp();
 
