@@ -8,16 +8,18 @@
  */
 
 
-#include<types.h>
-#include<lib.h>
-#include<proc.h>
-#include<lock.h>
-#include<mm.h>
-#include<time.h>
-#include<errno.h>
+#include<sys/types.h>
+#include<sys/param.h>
+#include<sys/mbuf.h>
+#include<kern/lib.h>
+#include<kern/proc.h>
+#include<kern/lock.h>
+#include<kern/mm.h>
+#include<kern/signal.h>
+#include<kern/errno.h>
 #include<net/net.h>
 #include<net/netlib.h>
-#include<net/ether.h>
+#include<net/ethernet.h>
 #include<net/arp.h>
 #include<net/icmp.h>
 #include<net/udp.h>
@@ -32,7 +34,7 @@
  **************************************************************************/
 
 
-//================================== ѥ֥å ======================================
+//================================== PROTECTED ==========================================
 
 
 /*
@@ -74,7 +76,7 @@ typedef struct{
 }HEAD_ID;
 
 
-//================================== ץ饤١ ======================================
+//================================== PRIVATE ============================================
 
 
 #define GET_ID(a)	enter_spinlock(&headId.lock);\
@@ -94,7 +96,7 @@ static void viewHead(IP_HEADER *head)
 *************************************************************************************************/
 
 
-//================================== Х ======================================
+//================================== PUBLIC =============================================
 
 
 /*
@@ -106,17 +108,14 @@ int transIp(SOCKET *sock, TRANS_BUF_INFO *tbi, uint dstip, uint16_t frag)
 {
 	char dstmac[6];
 	uchar tos;
-	uint transip;
-	int num;
-	int rest;
 	IP_HEADER head;
 	int max,total,trans;
-
-	/* Get device number. */
-	num = searchDevnum(dstip, &transip);
+	int rest;
+	int num = getEtherNum(dstip);
+	uint32_t transip = getTransIp(dstip, num);
 
 	/*  MACμ */
-	if ((rest = getMac(num, transip, dstmac)) < 0){
+	if ((rest = getMac(num, transip, dstmac)) != NOERR){
 		return rest;
 	}
 
@@ -134,14 +133,14 @@ int transIp(SOCKET *sock, TRANS_BUF_INFO *tbi, uint dstip, uint16_t frag)
 	GET_ID(head.id);
 	head.ttl = 255;
 	head.prot = tbi->ipType;
-	head.srcip = ethDev[num].ip;
+	head.srcip = getEtherIp(num);
 	head.dstip = dstip;
 	head.chksum = 0;
 
-	tbi->type = DIX_TYPE_IP;
+	tbi->type = ETHERTYPE_IP;
 	tbi->data[0] = (char*)&head;
 	tbi->size[0] = sizeof(IP_HEADER);			/* IP header size. */
-	max = ethDev[num].mtu - sizeof(IP_HEADER);	/* ǡ */
+	max = getEtherMtu(num) - sizeof(IP_HEADER);	/* ǡ */
 	total = tbi->size[1] + tbi->size[2];		/* ǡȡ륵 */
 
 	if (total <= max){
@@ -151,7 +150,7 @@ int transIp(SOCKET *sock, TRANS_BUF_INFO *tbi, uint dstip, uint16_t frag)
 /**********************************************
 viewHead(&head);
 **********************************************/
-		startEmu(ethDev[num].ifnet, tbi, dstmac);
+		transEther(getEtherIfnet(num), tbi, dstmac);
 	}
 	else{
 		head.len = swapWord(tbi->size[0] + max);
@@ -161,7 +160,7 @@ viewHead(&head);
 /**********************************************
 viewHead(&head);
 **********************************************/
-		startEmu(ethDev[num].ifnet, tbi, dstmac);
+		transEther(getEtherIfnet(num), tbi, dstmac);
 
 		trans = max;						/* ѥ */
 		tbi->data[2] += max-tbi->size[1];
@@ -179,7 +178,7 @@ viewHead(&head);
 /**********************************************
 viewHead(&head);
 **********************************************/
-				startEmu(ethDev[num].ifnet, tbi, dstmac);
+				transEther(getEtherIfnet(num), tbi, dstmac);
 				break;
 			}
 			else{
@@ -191,7 +190,7 @@ viewHead(&head);
 /**********************************************
 viewHead(&head);
 **********************************************/
-				startEmu(ethDev[num].ifnet, tbi, dstmac);
+				transEther(getEtherIfnet(num), tbi, dstmac);
 			}
 
 			trans += max;
@@ -221,14 +220,14 @@ enum{
 };
 
 typedef struct FRAG_HEAD{
-	int 	 use;		// ѺѤߥե饰
-	int		 curLen;	// ߤǡĹ
-	uint 	 creatTime; // 
+	int 	use;		// ѺѤߥե饰
+	int		curLen;		// ߤǡĹIPإåȴ
+	uint 	creatTime;	// 
 	char	*data;		// ǡǼХåե
 }FRAG_HEAD;
 
 
-//================================== ץ饤١ ======================================
+//================================== PRIVATE ============================================
 
 
 static char fragData[DATA_LEN_MAX];
@@ -360,7 +359,7 @@ static int reassemble(IP_HEADER *ip, FRAG_HEAD *fragHead)
 }
 
 
-//================================== ѥ֥å ======================================
+//================================== PROTECTED ==========================================
 
 
 /*
@@ -412,12 +411,17 @@ static void freeFragment(FRAG_HEAD *fragHead)
 }
 
 
-/*
- * IPǡ
- */
-static IP_HEADER *getIpFromFragment(FRAG_HEAD *fragHead)
+//աether_headerʬ
+// IPǡmbuf쥤ȤǼ
+// return : struct mbuf* or NULL
+static struct mbuf *getIpFromFragment(FRAG_HEAD *fragHead)
 {
-	return (IP_HEADER*)fragHead->data;
+	return m_devget(
+		fragHead->data, 
+		fragHead->curLen + sizeof(IP_HEADER) + sizeof(struct ether_header), 
+		sizeof(struct ether_header),
+		NULL, 
+		NULL);
 }
 
 
@@ -428,7 +432,7 @@ static IP_HEADER *getIpFromFragment(FRAG_HEAD *fragHead)
  **************************************************************************/
 
 
-//=================================== Х =======================================
+//================================== PUBLIC =============================================
 
 
 /*
@@ -436,9 +440,10 @@ static IP_HEADER *getIpFromFragment(FRAG_HEAD *fragHead)
  * parameters : IP header
  * return : 0 or task switch=1
  */
-int receiveIp(IP_HEADER *ip)
+int receiveIp(struct mbuf *mbuf)
 {
-	int (*receive)(IP_HEADER*);
+	IP_HEADER *ip = getMbufDataPointer(mbuf);
+	int (*receive)(struct mbuf*);
 
 /*****************************************************************************************
 TCP_HEADER *tcp = (TCP_HEADER*)ip->data;
@@ -449,7 +454,7 @@ printk("IP version=%x,ip length=%x,TOS=%x,total length=%x,id=%x,fragment=%x "
        swapWord(ip->fragOffset),ip->ttl,ip->prot,ip->srcip,ip->dstip);
 *******************************************************************************************/
 	// åγǧ
-	if (calcSum((uint*)ip,(ip->verhead&0xf) * 4)){
+	if (calcSum((uint*)ip, (ip->verhead & 0xf) * 4)){
 		return 0;
 	}
 
@@ -475,20 +480,23 @@ printk("IP version=%x,ip length=%x,TOS=%x,total length=%x,id=%x,fragment=%x "
 
 		// Υå
 		if (checkFragSize(ip) == ERR){
+			m_freem(mbuf);
 			return 0;
 		}
 		
 		// ե饰Ȥη
 		if (reassembleFragment(ip,&fragment) != REASSENBLED){
+			m_freem(mbuf);
 			return 0;
 		}
 
 		rest = receive(getIpFromFragment(fragment));
 		freeFragment(fragment);
+		m_freem(mbuf);
 		return rest;
 	}
 
-	return receive(ip);
+	return receive(mbuf);
 }
 
 
@@ -499,11 +507,11 @@ printk("IP version=%x,ip length=%x,TOS=%x,total length=%x,id=%x,fragment=%x "
  **************************************************************************/
 
 
-//================================== ץ饤١ ========================================
+//================================== PRIVATE ============================================
 
 
 static PROC *ipTask;
-static IP_BUFFER *recvBuf;
+static void *recvBuf;
 static int recvLock;
 
 
@@ -512,19 +520,21 @@ static int recvLock;
  */
 static inline void IpTask()
 {
-	IP_BUFFER *ipBuf;
+	struct mbuf *mbuf;
+	IP_HEADER *ip;
 
-	ipBuf = getRecvBuf(&recvBuf, &recvLock);
-	if (ipBuf == NULL){
-		sys_sleep(-1);
+	mbuf = getDetachRecvBuf(&recvBuf, &recvLock);
+	if (mbuf == NULL){
+		sigWait();
 	}
 	else{
-		switch (ipBuf->ip.prot){
+		ip = getMbufDataPointer(mbuf);
+		switch (ip->prot){
 			case IPPROTO_TCP:
-				doTcpService(ipBuf->sock, &ipBuf->ip);
+				doTcpService(mbuf->mh_sock, ip);
 				break;
 			case IPPROTO_UDP:
-				doUdpService(ipBuf->sock, &ipBuf->ip);
+				doUdpService(mbuf->mh_sock, ip);
 				break;
 			case IPPROTO_ICMP:
 				// ̤б
@@ -532,12 +542,12 @@ static inline void IpTask()
 			default:
 				break;
 		}
-		freeIpBuf(ipBuf);
+		m_freem(mbuf);
 	}
 }
 
 
-//================================== ѥ֥å ========================================
+//================================== PROTECTED ==========================================
 
 
 /*
@@ -552,7 +562,7 @@ static void startIpTask()
 			break;
 		case 0:
 			ipTask = get_current_task();
-			del_from_schedule(TASK_WAIT, PROC_SCHEDULE_DEL);
+			delFromSchedule(TASK_WAIT);
 			for (;;){
 				IpTask();
 			}
@@ -561,22 +571,16 @@ static void startIpTask()
 }
 
 
-//================================== Х ========================================
+//================================== PUBLIC =============================================
 
 
-/*
- * ! ա߻Τߤ˻Ѥ뤳
- * IPåɤε
- */
-void setIpTask(SOCKET *sock, IP_HEADER *ip)
+// IPåɤε
+//առ߻Τߤ˻Ѥ뤳
+void setIpTask(SOCKET *sock, struct mbuf *mbuf)
 {
-	IP_BUFFER *ipBuf;
-
-	if (allocIpBuf(ip, &ipBuf) == ERR){
-		return;
-	}
-	ipBuf->sock = sock;
-	attachRecvBuf(ipBuf, &recvBuf, &recvLock);
+	ASSERT(mbuf != NULL);
+	mbuf->mh_sock = sock;
+	attachRecvBuf(mbuf, &recvBuf, &recvLock);
 	sys_wake(ipTask);
 }
 
@@ -588,7 +592,7 @@ void setIpTask(SOCKET *sock, IP_HEADER *ip)
  **************************************************************************/
 
 
-//================================== Х ========================================
+//================================== PUBLIC =============================================
 
 
 int initIp()
