@@ -10,7 +10,8 @@
 #define ip_h
 
 
-#include<types.h>
+#include<sys/types.h>
+#include<sys/mbuf.h>
 #include<net/net.h>
 #include<netinet/in.h>
 #include<sys/ip.h>
@@ -37,8 +38,8 @@ typedef struct{
 
 
 extern int transIp(SOCKET*, TRANS_BUF_INFO*, uint, uint16_t);
-extern int receiveIp(IP_HEADER*);
-extern void setIpTask(SOCKET*, IP_HEADER*);
+extern int receiveIp(struct mbuf *);
+extern void setIpTask(SOCKET*, struct mbuf *);
 extern int initIp();
 
 
