@@ -8,17 +8,17 @@
  */
 
 
-#ifndef net_h
-#define net_h
+#ifndef ne_net_h
+#define ne_net_h
 
 
-#include<types.h>
-#include<proc.h>
-#include<fs.h>
-#include<mm.h>
+#include<sys/types.h>
 #include<sys/ip.h>
 #include<sys/socket.h>
 #include<netinet/in.h>
+#include<kern/proc.h>
+#include<kern/fs.h>
+#include<kern/mm.h>
 
 
 #ifndef ASM_FILE
@@ -37,7 +37,7 @@ enum{
 };
 
 
-typedef struct mbuf{
+typedef struct TRANS_BUF_INFO{
 	char  *data[3];		/* ե졼ॢɥ쥹 */
 	int    size[3];		/* ǡե졼ॵ */
 	ushort type;		/* ե졼ॿס */
@@ -67,34 +67,34 @@ typedef struct{
  *  fsscΰ֤ϲۥե빽¤Τΰ֤ȥ󥯤ƤΤա
  */
 typedef struct SOCKET{
-	F_DSC			 fd;			// եǥץ¤Ρ
+	F_DSC			fd;				// եǥץ¤Ρ
 	FS				*fs;			// FS¤Υɥ쥹
 	struct SOCKET	*sc;			// åȥɥ쥹VENTRY¤Τinodeб롣
-	uint16_t		 sockType;		// åȥס
-	uint16_t		 stat;			// ͥ֡
+	uint16_t		sockType;		// åȥס
+	uint16_t		stat;			// ͥ֡
 	void			*sinf;			// SOCKET_INFO.
-	in_port_t		 srcport;		// Source port number.
-	in_port_t		 dstport;		// Connection port number.
-	in_addr_t		 dstip;			// Connection IP address.
-	in_addr_t		 srcip;			// Local host IP address.
+	in_port_t		srcport;		// Source port number.
+	in_port_t		dstport;		// Connection port number.
+	in_addr_t		dstip;			// Connection IP address.
+	in_addr_t		srcip;			// Local host IP address.
 	void			*recvBuf;		// Receive buffer address.
 	PROC			*waitProc;		// Interrupt wait queue.
-	uint32_t		 seqnum;		// Sequence number.
-	uint32_t		 acknum;		// Ack number.
-	uint32_t		 prevAck;		// send()ǥǡȤֹ򵭲Ƥ
+	uint32_t		seqnum;			// Sequence number.
+	uint32_t		acknum;			// Ack number.
+	uint32_t		prevAck;		// send()ǥǡȤֹ򵭲Ƥ
 	struct SOCKET	*next;			// ͥ󥯡
 	struct SOCKET	*prev;			// 
 	struct SOCKET	*waitNext;		// ͥǽԤ󥯡
-	uint			 waitTime;		// ǽԤϻms
-	uint16_t		 mss;			// ͥȺ祵
-	uint16_t		 window;		// ͥ襦ɥ
-	SEND_BUF		 sendBuf;		// Хåե¤
+	uint			waitTime;		// ǽԤϻms
+	uint16_t		mss;			// ͥȺ祵
+	uint16_t		window;			// ͥ襦ɥ
+	SEND_BUF		sendBuf;		// Хåե¤
 	void			*ringBuf;		// ǡ¸ѥ󥰥Хåե
 	PROC			*listenThread;	// å󥹥å
 	struct SOCKET	*newsc_next;	// å󥳥ͥѥåȥ
-	int				 lockGate;		// ԥå
-	SOCK_OPT		 sockOpt;		// åȥץ
-	IP_OPT			 ipOpt;			// IPץ
+	int				lockGate;		// ԥå
+	SOCK_OPT		sockOpt;		// åȥץ
+	IP_OPT			ipOpt;			// IPץ
 }SOCKET;
 
 typedef struct{
