@@ -8,29 +8,29 @@
  */
 
 
-#include<config.h>
-#include<types.h>
-#include<lib.h>
-#include<errno.h>
-#include<mm.h>
-#include<proc.h>
-#include<time.h>
-#include<lock.h>
-#include<interrupt.h>
-#include<signal.h>
-#include<fs.h>
-#include<console.h>
+#include<sys/config.h>
+#include<sys/types.h>
+#include<sys/param.h>
+#include<kern/lib.h>
+#include<kern/errno.h>
+#include<kern/mm.h>
+#include<kern/proc.h>
+#include<kern/time.h>
+#include<kern/lock.h>
+#include<kern/interrupt.h>
+#include<kern/signal.h>
+#include<kern/fs.h>
+#include<dev/console/console.h>
 #include<net/net.h>
-#include<net/ether.h>
+#include<net/ethernet.h>
 #include<net/ip.h>
 #include<net/tcp.h>
 #include<net/netlib.h>
-#include<debug.h>
-#include<test.h>
+#include<kern/debug.h>
+#include<kern/test.h>
 
 
 //#define DEBUG_TCP 1
-
 #ifdef DEBUG_TCP
 	#define STATIC
 	#define INLINE
@@ -47,6 +47,9 @@
  **************************************************************************/
 
 
+//================================== PROTECTED ==========================================
+
+
 enum{
 	/* TCP flag. */
 	TCP_FLG_FIN = 1<<0,
@@ -67,11 +70,7 @@ typedef struct{
 }MSS;
 
 
-//================================== ѥ֥å ========================================
-
-
 /*
- * PUBLIC
  * إåꡣ
  * parameters : dectination IP,pseudo header address,data size
  */
@@ -86,19 +85,17 @@ STATIC INLINE void setPseudoHead(in_addr_t dstip,in_addr_t srcip,uint16_t size,P
 
 
 /*
- * PUBLIC
  * Get sequence number.
  * return : sequence number
  */
 STATIC INLINE uint32_t getSeqNumber()
 {
 	/* 4msǣ䤹RFC793ˡ */
-	return rdtsc()/clock_1m*4;
+	return calcTimeFromClock(rdtsc()) * 4;
 }
 
 
 /*
- * PUBLIC
  * TCPХȿ֤
  */
 STATIC INLINE int getTcpSize(IP_HEADER *ip)
@@ -108,14 +105,12 @@ STATIC INLINE int getTcpSize(IP_HEADER *ip)
 
 
 /*
- * PUBLIC
  * TCPǡХȿ֤
  */
 STATIC int getTcpDataSize(IP_HEADER *ip)
 {
 	TCP_HEADER *tcp = (TCP_HEADER*)ip->data;
 
-
 	return getTcpSize(ip) - (tcp->headlen >> 4) * 4;
 }
 
@@ -147,7 +142,7 @@ typedef struct{
 }OPTIONS;
 
 
-//================================== ѥ֥å ========================================
+//================================== PROTECTED ==========================================
 
 
 /*
@@ -158,32 +153,29 @@ STATIC void getTcpOptions(uchar *c, int size, OPTIONS *optst)
 {
 	uchar *last;
 
-
 	last = c + size;
-	while (c < last)
-	{
-		switch (*c)
-		{
-			case TCP_KIND_MSS:
-				optst->mss = *(uint16_t*)(c + 2);
-				c += 4;
-				break;
-			case TCP_KIND_WSC:
-				optst->winscal = *(uint8_t*)(c + 2);
-				c += 3;
-				break;
-			case TCP_KIND_TSTMP:
-				optst->timeStamp = *(uint32_t*)(c + 2);
-				optst->timeEcho = *(uint32_t*)(c + 6);
-				c += 10;
-				break;
-			case TCP_KIND_NOP:
-				c += 1;
-				break;
-			case TCP_KIND_END:
-				break;
-			default:
-				c += *(c + 1);
+	while (c < last){
+		switch (*c){
+		case TCP_KIND_MSS:
+			optst->mss = *(uint16_t*)(c + 2);
+			c += 4;
+			break;
+		case TCP_KIND_WSC:
+			optst->winscal = *(uint8_t*)(c + 2);
+			c += 3;
+			break;
+		case TCP_KIND_TSTMP:
+			optst->timeStamp = *(uint32_t*)(c + 2);
+			optst->timeEcho = *(uint32_t*)(c + 6);
+			c += 10;
+			break;
+		case TCP_KIND_NOP:
+			c += 1;
+			break;
+		case TCP_KIND_END:
+			break;
+		default:
+			c += *(c + 1);
 		}
 	}
 }
@@ -201,7 +193,7 @@ enum{
 };
 
 
-//================================== ѥ֥å ========================================
+//================================== PROTECTED ==========================================
 
 
 // Transmit a tcp segment.
@@ -220,7 +212,7 @@ STATIC void transSegment(
 	TCP_HEADER *send_tcp = (TCP_HEADER*)(head + sizeof(PSEUDO_HEADER));
 
 	// إå
-	setPseudoHead(sock->dstip, getSrcip(sock->dstip), sizeof(TCP_HEADER) + opt_size+data_size, (PSEUDO_HEADER*)head);
+	setPseudoHead(sock->dstip, getEtherIp(getEtherNum(sock->dstip)), sizeof(TCP_HEADER) + opt_size+data_size, (PSEUDO_HEADER*)head);
 
 	// Хåեơ֥
 	tbi.data[2] = data;
@@ -372,54 +364,63 @@ STATIC int sendFromBuf(SOCKET *sock, const size_t i_sendStart, const uint8_t tra
 static int waitCnctLock;	/* Ԥѥåȡ */
 
 
-//================================== ץ饤١ ========================================
+//================================== PRIVATE ============================================
 
 
 static SOCKET waitCnctSc;	/* Ԥ󥯡 */
 static SOCKET waitLstnSc;	/* å󥯡 */
 
 
-/*
- *  ߽ؿ 
- * ԤåȤõ
- * return : 0 = ̾ｪλ 1 = 򵯤 -1 = 顼
- */
-STATIC int searchSocket(IP_HEADER *ip)
+//Գ
+// ԤåȤõ
+// return : 0 = ̾ｪλ 1 = 򵯤 -1 = 顼
+STATIC int searchSocket(struct mbuf *mbuf)
 {
 	SOCKET *sock;
-	IP_BUFFER *ipBuf;
+	IP_HEADER *ip = getMbufDataPointer(mbuf);
 	TCP_HEADER *tcp = (TCP_HEADER*)ip->data;
+/***************************************************************************
+enum{COUNT = 7,};
+static int cnt = 0;
+***************************************************************************/
 
-/****************************************************************************************
-union{
-	uint ui;
-	uchar uc[4];
-}srcIp;
-
-if ((ip->srcip == 0x20019ac) && (swapWord(tcp->srcport) == 20)){
-	srcIp.ui = ip->srcip;
-	printk("searchSocket() %u.%u.%u.%u srcport=%u dstport=%u seq=%x ",
-		srcIp.uc[0],srcIp.uc[1],srcIp.uc[2],srcIp.uc[3],
-		swapWord(tcp->srcport),swapWord(tcp->dstport),
-		swapInt32(tcp->seqnum));
-}
-****************************************************************************************/
 	/* ͥ󥯤õ */
 	for (sock = waitCnctSc.next; sock != &waitCnctSc; sock = sock->next){
 		if ((sock->srcport == tcp->dstport) && (sock->dstport == tcp->srcport) && (sock->dstip == ip->srcip)){
+/****************************************************************************************
+{
+	union{
+		uint ui;
+		uchar uc[4];
+	}srcIp;
+
+	if ((ip->srcip == 0x40019ac) && (swapWord(tcp->srcport) == 20) && (++cnt <= COUNT)){
+		srcIp.ui = ip->srcip;
+		printk(
+			"searchSocket() "
+//			"%u.%u.%u.%u "
+//			"srcport=%u "
+//			"dstport=%u "
+			"seq=%x "
+			"sockAck=%x\n",
+//			srcIp.uc[0],srcIp.uc[1],srcIp.uc[2],srcIp.uc[3],
+//			swapWord(tcp->srcport),
+//			swapWord(tcp->dstport),
+			swapInt32(tcp->seqnum),
+			sock->acknum);
+	}
+}
+****************************************************************************************/
 			/* åȤϴǤƤ롣 */
 			if (sock->stat & SOCK_DISCON_RECV){
-				setIpTask(sock,ip);
+				setIpTask(sock, mbuf);
 				return 1;
 			}
 
 			// ǡ򥽥åȤϤ
-			if (allocIpBuf(ip, &ipBuf) == ERR){
-				return 0;
-			}
-			attachRecvBufTcp(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
+			attachRecvBufTcp(mbuf, &sock->recvBuf, &sock->lockGate);
 			if (sock->waitProc != NULL){
-				sys_wake(sock->waitProc);
+				sigWakeSoon(sock->waitProc);
 			}
 			return 1;
 		}
@@ -428,53 +429,35 @@ if ((ip->srcip == 0x20019ac) && (swapWord(tcp->srcport) == 20)){
 	/* å󥯤õ */
 	for (sock = waitLstnSc.next; sock != &waitLstnSc; sock = sock->next){
 		if ((sock->srcport == tcp->dstport) && ((sock->srcip == INADDR_ANY) || (sock->srcip == ip->dstip))){
-			if (allocIpBuf(ip,&ipBuf) == ERR){
-				return 0;
-			}
-			attachRecvBufTcp(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
+			attachRecvBufTcp(mbuf, &sock->recvBuf, &sock->lockGate);
 			sys_wake(sock->listenThread);
 			return 1;
 		}
 	}
 	
 	// Ĥ餺
-	setIpTask(NULL, ip);
-	
+	setIpTask(NULL, mbuf);
+
 	return 0;
 }
 
 
-//================================== ѥ֥å ========================================
+//================================== PROTECTED ==========================================
 
 
-/*
- * ǡ
- * return : 0 or error number
- */
-STATIC int waitReceive(SOCKET *sock,uint timeout)
+// ǡ
+// return : error number
+STATIC int waitReceive(SOCKET *sock, uint timeout)
 {
-	for (;;)
-	{
-		IP_BUFFER *ipBuf;
-		int rest;
-		
-		ipBuf = refIpBuf(sock->recvBuf);
-		if (ipBuf == NULL){
-			sock->waitProc = get_current_task();
-			rest = sys_sleep(timeout);
-		}
-		else if (ipBuf->access == 1){
-			ipBuf->access = 0;
-			sock->waitProc = get_current_task();
-			rest = sys_sleep(timeout);
-		}
-		else{
-			break;
-		}
+	struct mbuf *mbuf;
+	int restTime;
 
-		/* Time out. */
-		if (rest == 0){
-			if (refIpBuf(sock->recvBuf) == NULL){
+	for (mbuf = refRecvBuf(sock->recvBuf); (mbuf == NULL) || (mbuf->mh_access == YES); mbuf = refRecvBuf(sock->recvBuf)){
+		mbuf->mh_access = NO;
+		sock->waitProc = get_current_task();
+		restTime = sys_sleep(timeout);
+		if (restTime == 0){			/* Time out. */
+			if (refRecvBuf(sock->recvBuf) == NULL){
 				return -ETIMEDOUT;
 			}
 		}
@@ -486,7 +469,7 @@ STATIC int waitReceive(SOCKET *sock,uint timeout)
 		}
 	}
 
-	return 0;
+	return NOERR;
 }
 
 
@@ -496,7 +479,9 @@ STATIC int waitReceive(SOCKET *sock,uint timeout)
  */
 STATIC void addWaitCnctLink(SOCKET *sc)
 {
-	cli();
+	int eflag;
+
+	eflag = enterCli();
 	enter_spinlock(&waitCnctLock);
 	{
 		sc->next = waitCnctSc.next;
@@ -505,7 +490,7 @@ STATIC void addWaitCnctLink(SOCKET *sc)
 		waitCnctSc.next = sc;
 	}
 	exit_spinlock(&waitCnctLock);
-	sti();
+	exitCli(eflag);
 }
 
 
@@ -515,7 +500,9 @@ STATIC void addWaitCnctLink(SOCKET *sc)
  */
 STATIC void addWaitLstnLink(SOCKET *sc)
 {
-	cli();
+	int eflag;
+
+	eflag = enterCli();
 	enter_spinlock(&waitCnctLock);
 	{
 		sc->next=waitLstnSc.next;
@@ -524,7 +511,7 @@ STATIC void addWaitLstnLink(SOCKET *sc)
 		waitLstnSc.next=sc;
 	}
 	exit_spinlock(&waitCnctLock);
-	sti();
+	exitCli(eflag);
 }
 
 
@@ -534,7 +521,9 @@ STATIC void addWaitLstnLink(SOCKET *sc)
  */
 STATIC void delWaitLink(SOCKET *sc)
 {
-	cli();
+	int eflag;
+
+	eflag = enterCli();
 	enter_spinlock(&waitCnctLock);
 	{
 		sc->next->prev = sc->prev;
@@ -542,7 +531,7 @@ STATIC void delWaitLink(SOCKET *sc)
 		sc->next = sc->prev = sc;
 	}
 	exit_spinlock(&waitCnctLock);
-	sti();
+	exitCli(eflag);
 }
 
 
@@ -554,9 +543,9 @@ STATIC void delWaitLink(SOCKET *sc)
 STATIC SOCKET *searchListenSocket(in_port_t srcport)
 {
 	SOCKET *sc,*search_sc = NULL;
-	
-	
-	cli();
+	int eflag;
+
+	eflag = enterCli();
 	enter_spinlock(&waitCnctLock);
 	{
 		for (sc = waitLstnSc.next; sc != &waitLstnSc; sc = sc->next)
@@ -567,7 +556,7 @@ STATIC SOCKET *searchListenSocket(in_port_t srcport)
 			}
 	}
 	exit_spinlock(&waitCnctLock);
-	sti();
+	exitCli(eflag);
 	
 	return search_sc;
 }
@@ -625,33 +614,30 @@ STATIC int isAck(TCP_HEADER *tcp, uint32_t sent, uint32_t received)
  * ǡ
  * 쥷֥Хåեϥ󥯤ڤΥʤ
  * parameters : socket,쥷֥Хåեݥ,send flag
- * return : 0 or error number
+ * return : error number
  */
 STATIC int recvData(SOCKET *sock, const uint timeout)
 {
-	IP_BUFFER *ipBuf;
-	TCP_HEADER *tcp;
-	int rest;
-	
+	int error;
 	
 	for (;;){
+		struct mbuf *mbuf;
+		IP_HEADER *ip;
+		TCP_HEADER *tcp;
+
 		// Ԥ
-		rest = waitReceive(sock,timeout);
-		if (rest < 0){
-			return rest;
+		error = waitReceive(sock, timeout);
+		if (error != NOERR){
+			return error;
 		}
+		mbuf = refRecvBuf(sock->recvBuf);
+		ip = getMbufDataPointer(mbuf);
+		tcp = (TCP_HEADER*)ip->data;
 
-		ipBuf = refIpBuf(sock->recvBuf);
-		tcp = (TCP_HEADER*)ipBuf->ip.data;
-
-/****************************************************************************************************************/
-//printDebug(80 * 9 + 58, "0x%x 0x%x",swapInt32(tcp->seqnum), sock->acknum);
-/****************************************************************************************************************/
 		// ꥻåȥե饰ʤ饳ͥ
-		if (tcp->flag & TCP_FLG_RST)
-		{
-			detachRecvBuf(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-			freeIpBuf(ipBuf);
+		if (tcp->flag & TCP_FLG_RST){
+			detachRecvBuf(mbuf, &sock->recvBuf, &sock->lockGate);
+			m_freem(mbuf);
 			return -ECONNRESET;
 		}
 
@@ -662,16 +648,19 @@ STATIC int recvData(SOCKET *sock, const uint timeout)
 				return 0;
 			case ACK_EQ_SEQ_GR:
 				// ΥȤ줿ǽΤǱƤ
-				ipBuf->access = 1;
+				mbuf->mh_access = YES;
 				if (tcp->flag & TCP_FLG_FIN){
 					sendFinAck(sock);
 				}
 				else{
 					transSegment(sock, TCP_FLG_ACK, NULL, 0, NULL, 0, TCP_WIN_SIZE);
-					
-					// IPХåե̵ϥåȤαIPХåեĳ롣
-					if (isIpBufFree() == FALSE){
-						freeRecvBuf(sock);
+					if (isNoRecvBuf() == YES){
+						// IPХåե̵ϥåȤαIPХåեĳ롣
+						struct mbuf *end = refRecvEndBuf(sock->recvBuf);
+						if (end != NULL){
+							detachRecvBuf(end, sock->recvBuf, &sock->lockGate);
+							m_freem(end);
+						}
 					}
 				}
 				break;
@@ -685,12 +674,12 @@ STATIC int recvData(SOCKET *sock, const uint timeout)
 				}
 				//
 			default:
-				detachRecvBuf(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-				freeIpBuf(ipBuf);
+				detachRecvBuf(mbuf, &sock->recvBuf, &sock->lockGate);
+				m_freem(mbuf);
 	    }
     }
 
-	return 0;
+	return NOERR;
 }
 
 
@@ -703,40 +692,41 @@ STATIC int recvData(SOCKET *sock, const uint timeout)
 enum{RECV_SEND_GET_DATA = 1,};
 STATIC int recvSend(SOCKET *sock, const uint timeout, size_t *o_tcpAck)
 {
-	int rest;
+	int error;
 
 	if (0 < timeout){
-		rest = waitReceive(sock,timeout);
-		if (rest < 0){
-			return rest;
+		error = waitReceive(sock, timeout);
+		if (error != NOERR){
+			return error;
 		}
 	}
 
 	for (;;){
-		IP_BUFFER *ipBuf;
+		struct mbuf *mbuf;
+		IP_HEADER *ip;
 		TCP_HEADER *tcp;
-		
-		ipBuf = refIpBuf(sock->recvBuf);
-		if (ipBuf == NULL){
+
+		mbuf = refRecvBuf(sock->recvBuf);
+		if (mbuf == NULL){
 			break;
 		}
-		tcp = (TCP_HEADER*)ipBuf->ip.data;
+		ip = getMbufDataPointer(mbuf);
+		tcp = (TCP_HEADER*)ip->data;
 
-		if (tcp->flag & TCP_FLG_RST)
-		{
-			detachRecvBuf(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-			freeIpBuf(ipBuf);
+		// ꥻåȥե饰ʤ饳ͥ
+		if (tcp->flag & TCP_FLG_RST){
+			detachRecvBuf(mbuf, &sock->recvBuf, &sock->lockGate);
+			m_freem(mbuf);
 			return -ECONNRESET;
 		}
 
-		switch (isAck(tcp, sock->seqnum, sock->acknum))
-		{
+		switch (isAck(tcp, sock->seqnum, sock->acknum)){
 			case ACK_EQ_SEQ_EQ:
 				// 
 			case ACK_LW_SEQ_EQ:
 				*o_tcpAck = swapInt32(tcp->acknum);
 				sock->window = swapWord(tcp->wndsize);
-				if (0 < getTcpDataSize(&ipBuf->ip)){
+				if (0 < getTcpDataSize(ip)){
 					// ǡ硢쥷ִؿϤѥåȤĤƤ
 					// ϤѥåȤĤäƤ롣
 					if (sock->prevAck != tcp->seqnum){
@@ -750,8 +740,8 @@ STATIC int recvSend(SOCKET *sock, const uint timeout, size_t *o_tcpAck)
 				}
 				// 
 			default:
-				detachRecvBuf(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-				freeIpBuf(ipBuf);
+				detachRecvBuf(mbuf, &sock->recvBuf, &sock->lockGate);
+				m_freem(mbuf);
 	    }
     }
 
@@ -769,31 +759,30 @@ STATIC void initReceive()
 }
 
 
-//================================== Х ========================================
+//================================== PUBLIC =============================================
 
 
-/*
- *  ߽ؿ 
- * return : 0 or task switch=1
- */
-int receiveTcp(IP_HEADER *ip)
+//Գ
+// return : 0 or task switch=1
+int receiveTcp(struct mbuf *mbuf)
 {
+	IP_HEADER *ip = getMbufDataPointer(mbuf);
 	int tcpSize;
 	PSEUDO_HEADER pseudoHead;
 
-
 	/* åγǧ */
 	tcpSize = getTcpSize(ip);
-	setPseudoHead(ip->dstip,ip->srcip,tcpSize,&pseudoHead);
-	if (calcSumM((uint*)&pseudoHead,(uint*)ip->data,sizeof(PSEUDO_HEADER),tcpSize))
+	setPseudoHead(ip->dstip, ip->srcip, tcpSize, &pseudoHead);
+	if (calcSumM((uint*)&pseudoHead, (uint*)ip->data, sizeof(PSEUDO_HEADER), tcpSize)){
 		return 0;
+	}
 	
 	/* γǧ */
 	if (TCP_MSS_SIZE < getTcpDataSize(ip)){
 		return 0;
 	}
 
-	return searchSocket(ip);
+	return searchSocket(mbuf);
 }
 
 
@@ -808,7 +797,7 @@ enum{
 };
 
 
-//================================== ץ饤١ ========================================
+//================================== PRIVATE ============================================
 
 
 /*
@@ -822,13 +811,14 @@ STATIC void addNewSocket(SOCKET *sc, SOCKET *newsc)
 	SOCKET **p;
 	SOCKET *delSock[DEL_NUM];
 	int delNum = 0;
-	int i;
 	uint currentTime = sys_time(NULL);
+	int eflag;
+	int i;
 	
 	// åȤǸ³
 	newsc->waitTime = currentTime;
 	newsc->newsc_next = NULL;
-	cli();
+	eflag = enterCli();
 	enter_spinlock(&sc->lockGate);
 	{
 		for (p = &sc->newsc_next;; p = &(*p)->newsc_next){
@@ -838,7 +828,7 @@ STATIC void addNewSocket(SOCKET *sc, SOCKET *newsc)
 			}
 
 			// ֿॢȤΥåȤ
-			if ((LISTEN_WAIT_MAX < currentTime - (*p)->waitTime) && (refIpBuf(sc->recvBuf) != NULL)){
+			if ((LISTEN_WAIT_MAX < currentTime - (*p)->waitTime) && (refRecvBuf(sc->recvBuf) != NULL)){
 				if (delNum < DEL_NUM){
 					delSock[delNum] = *p;
 					*p = (*p)->newsc_next;
@@ -852,7 +842,7 @@ STATIC void addNewSocket(SOCKET *sc, SOCKET *newsc)
 		}
 	}
 	exit_spinlock(&sc->lockGate);
-	sti();
+	exitCli(eflag);
 	
 	// ֿॢȥåȤ򥳥ͥ󥯤
 	for (i = 0; i < delNum; ++i){
@@ -865,32 +855,32 @@ STATIC void addNewSocket(SOCKET *sc, SOCKET *newsc)
 
 /*
  * åȤ
- * return :  0 or error number
+ * return :  error number
  */
-STATIC int makeNewSocket(SOCKET *sock,IP_BUFFER *ipBuf,SOCKET **o_newSock)
+STATIC int makeNewSocket(SOCKET *sock, struct mbuf *mbuf, SOCKET **o_newSock)
 {
 	SOCKET *newSock;
 	IP_HEADER *ip;
 	TCP_HEADER *tcp;
 	OPTIONS optst;
-	int rest;
+	int error;
 
+	error = makeSocket(sock->sockType, IPPROTO_TCP, &newSock);
+	if (error < 0){
+		return error;
+	}
 
-	rest = makeSocket(sock->sockType,IPPROTO_TCP,&newSock);
-	if (rest < 0)
-		return rest;
-
-	ip = &ipBuf->ip;
+	ip = getMbufDataPointer(mbuf);;
 	tcp = (TCP_HEADER*)ip->data;
 
 	/* ץ(MSSΤ߼) */
 	optst.mss = TCP_MSS_SIZE;
-	getTcpOptions(tcp->option,getTcpDataSize(ip),&optst);
+	getTcpOptions(tcp->option, getTcpDataSize(ip), &optst);
 
 	/*
 	 * åȤꡣ
 	 */
-	memcpy(&newSock->fs,&sock->fs,sizeof(SOCKET)-OFFSETOF(SOCKET,fs));
+	memcpy(&newSock->fs, &sock->fs, sizeof(SOCKET) - OFFSETOF(SOCKET, fs));
 	newSock->dstip			= ip->srcip;
 	newSock->dstport		= tcp->srcport;
 	newSock->srcport		= tcp->dstport;
@@ -906,7 +896,7 @@ STATIC int makeNewSocket(SOCKET *sock,IP_BUFFER *ipBuf,SOCKET **o_newSock)
 
 	*o_newSock = newSock;
 	
-	return 0;
+	return NOERR;
 }
 
 
@@ -915,31 +905,33 @@ STATIC int makeNewSocket(SOCKET *sock,IP_BUFFER *ipBuf,SOCKET **o_newSock)
  */
 STATIC void listenConnect(SOCKET *sock)
 {
-	IP_BUFFER *ipBuf = getRecvBuf((IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-	IP_HEADER *ip = &ipBuf->ip;
-	TCP_HEADER *tcp = (TCP_HEADER*)ip->data;
+	struct mbuf *mbuf = getDetachRecvBuf(&sock->recvBuf, &sock->lockGate);
+	IP_HEADER *ip;
+	TCP_HEADER *tcp;
 	MSS mss;
 	SOCKET *newSock;
 	
+	ASSERT(mbuf != NULL);
+	ip = getMbufDataPointer(mbuf);
+	tcp = (TCP_HEADER*)ip->data;
 
 	/* ³׵᥻Ȥγǧ */
-	if ((tcp->flag & TCP_FLG_SYN) && (tcp->acknum == 0))
+	if ((tcp->flag & TCP_FLG_SYN) && (tcp->acknum == 0)){
 		;
-	else
-	{
-		sendReset(&ipBuf->ip);
-		freeIpBuf(ipBuf);
+	}
+	else{
+		sendReset(ip);
+		m_freem(mbuf);
 		return;
 	}
 
 	/* åȤκ */
-	if (makeNewSocket(sock,ipBuf,&newSock) < 0)
-	{
-		sendReset(&ipBuf->ip);
-		freeIpBuf(ipBuf);
+	if (makeNewSocket(sock, mbuf, &newSock) != NOERR){
+		sendReset(ip);
+		m_freem(mbuf);
 		return;
 	}
-	freeIpBuf(ipBuf);
+	m_freem(mbuf);
 	
 	/* ͥ󥯤³ */
 	addWaitCnctLink(newSock);
@@ -953,7 +945,7 @@ STATIC void listenConnect(SOCKET *sock)
 	transSegment(newSock, TCP_FLG_SYN|TCP_FLG_ACK, &mss, sizeof(mss), NULL, 0, TCP_WIN_SIZE);
 
 	/* å󥽥åȤ³ */
-	addNewSocket(sock,newSock);
+	addNewSocket(sock, newSock);
 }
 
 
@@ -966,9 +958,9 @@ STATIC void listenRoutin(SOCKET *sock)
 	addWaitLstnLink(sock);
 
 	for (;;){
-		if (refIpBuf(sock->recvBuf) == NULL){
+		if (refRecvBuf(sock->recvBuf) == NULL){
 			/* ꡼ */
-			sys_sleep(-1);
+			sigWait();
 
 			/* å֤ǤʤХåɽλ */
 			if ((sock->stat & SOCK_LISTEN) == 0){
@@ -986,7 +978,7 @@ STATIC void listenRoutin(SOCKET *sock)
 }
 
 
-//================================== ѥ֥å ========================================
+//================================== PROTECTED ==========================================
 
 
 /*
@@ -997,13 +989,13 @@ STATIC void listenRoutin(SOCKET *sock)
 STATIC SOCKET *getFromListenSocket(SOCKET *sc)
 {
 	SOCKET **p,*rest = NULL;
+	int eflag;
 	
-	
-	cli();
+	eflag = enterCli();
 	enter_spinlock(&sc->lockGate);
 	{
 		for (p = &sc->newsc_next; *p != NULL; p = &(*p)->newsc_next){
-			if (refIpBuf((*p)->recvBuf) != NULL){
+			if (refRecvBuf((*p)->recvBuf) != NULL){
 				rest = *p;
 				*p = rest->newsc_next;
 				rest->waitTime = 0;
@@ -1012,7 +1004,7 @@ STATIC SOCKET *getFromListenSocket(SOCKET *sc)
 		}
 	}
 	exit_spinlock(&sc->lockGate);
-	sti();
+	exitCli(eflag);
 	
 	return rest;
 }
@@ -1025,11 +1017,9 @@ STATIC void releaseListenSocket(SOCKET *sc)
 {
 	SOCKET *p,*q;
 	
-	
-	for (p = sc->newsc_next; p != NULL; p = q)
-	{
+	for (p = sc->newsc_next; p != NULL; p = q){
 		q = p->newsc_next;
-		releaseIpBuf(p->recvBuf);
+		releaseRecvBuf(p->recvBuf);
 		kfree(p);
 	}
 }
@@ -1040,11 +1030,10 @@ STATIC void releaseListenSocket(SOCKET *sc)
  * parameters : socket,file discripter
  * return     : 0 or error number
  */
-STATIC int makeListenThread(SOCKET *sock,int fd)
+STATIC int makeListenThread(SOCKET *sock, int fd)
 {
 	/* listenѥץư */
 	switch(forkKernelThread())
-//	switch(sys_fork())
     {
 		case -1:
 			return -ENOBUFS;
@@ -1084,7 +1073,7 @@ STATIC void endListenThread(SOCKET *sock)
  **************************************************************************/
 
 
-//================================== ѥ֥å ==========================================
+//================================== PROTECTED ==========================================
 
 
 // Хåե˻ĤäƤǡ롣
@@ -1151,7 +1140,7 @@ enum{
 };
 
 
-//================================== ץ饤١ ========================================
+//================================== PRIVATE ============================================
 
 
 static SOCKET *finSocket;		/* 楽åȥ󥯡 */
@@ -1185,7 +1174,7 @@ STATIC void delCloseTimeout()
 		{
 			p = finSocket->waitNext;
 			delWaitLink(finSocket);
-			releaseIpBuf(finSocket->recvBuf);
+			releaseRecvBuf(finSocket->recvBuf);
 			kfree(finSocket);
 			finSocket = p;
 		}
@@ -1195,7 +1184,7 @@ STATIC void delCloseTimeout()
 }
 
 
-//================================== ѥ֥å ========================================
+//================================== PROTECTED ==========================================
 
 
 /*
@@ -1226,12 +1215,12 @@ STATIC void disconnect(SOCKET *sock)
  */
 STATIC int connectTo(SOCKET *sock)
 {
-	int rest;
-	MSS mss;
-	IP_BUFFER *ipBuf = NULL;
+	struct mbuf *mbuf = NULL;
 	IP_HEADER *ip;
 	TCP_HEADER *tcp;
 	OPTIONS optst;
+	MSS mss;
+	int error;
 
 	/* ͥ󥯤³ */
 	addWaitCnctLink(sock);
@@ -1246,25 +1235,26 @@ STATIC int connectTo(SOCKET *sock)
 	sock->seqnum += 1;
 
 	/* Ȥμ */
-	rest = waitReceive(sock,REPLY_TIMEOUT);
-	if (rest < 0){
+	error = waitReceive(sock, REPLY_TIMEOUT);
+	if (error != NOERR){
 		goto ERR;
 	}
-	ipBuf = getRecvBuf((IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-	ip = &ipBuf->ip;
+	mbuf = getDetachRecvBuf(&sock->recvBuf, &sock->lockGate);
+	ASSERT(mbuf != NULL);
+	ip = getMbufDataPointer(mbuf);
 	tcp = (TCP_HEADER*)ip->data;
-	if (((tcp->flag & TCP_FLG_SYN) == 0) || (isAck(tcp,sock->seqnum,0) != ACK_EQ_SEQ_GR)){
+	if (((tcp->flag & TCP_FLG_SYN) == 0) || (isAck(tcp, sock->seqnum, 0) != ACK_EQ_SEQ_GR)){
 		if (tcp->flag & TCP_FLG_RST){
 			return -ECONNREFUSED;
 		}
 		transReset(sock, tcp);
-		rest = -EAGAIN;
+		error = -EAGAIN;
 		goto ERR;
 	}
 
 	/* ץγǧ(MSSΤ߼) */
-	optst.mss=TCP_MSS_SIZE;
-	getTcpOptions(tcp->option,getTcpDataSize(ip),&optst);
+	optst.mss = TCP_MSS_SIZE;
+	getTcpOptions(tcp->option, getTcpDataSize(ip), &optst);
 	sock->mss = optst.mss;
 
 	/* ǧ */
@@ -1275,25 +1265,26 @@ STATIC int connectTo(SOCKET *sock)
 
 	sock->sendBuf.buf = kmalloc(sock->mss);
 	if (sock->sendBuf.buf == NULL){
-		return -ENOMEM;
+		error = -ENOMEM;
+		goto ERR;
 	}
 	sock->sendBuf.curLen = 0;
-	rest = allocRingBuf(SEND_BUF_SIZE, sock->seqnum, &sock->ringBuf);
-	if (rest < 0){
-		return rest;
+	error = allocRingBuf(SEND_BUF_SIZE, sock->seqnum, &sock->ringBuf);
+	if (error < 0){
+		goto ERR;
 	}
 
-	freeIpBuf(ipBuf);
+	m_freem(mbuf);
 
 	return 0;
 ERR:
-	freeIpBuf(ipBuf);
+	m_freem(mbuf);
 
 	/* ͥ󥯤 */
 	delWaitLink(sock);
-	releaseIpBuf(sock->recvBuf);
+	releaseRecvBuf(sock->recvBuf);
 
-	return rest;
+	return error;
 }
 
 
@@ -1304,7 +1295,7 @@ ERR:
  **************************************************************************/
 
 
-/*================================== Х ========================================*/
+//================================== PUBLIC =============================================
 
 
 /*
@@ -1321,13 +1312,6 @@ void doTcpService(SOCKET *sock, IP_HEADER *ip)
 }
 
 
-/**************************************************************************
- *
- * åȥӥؿ
- *
- **************************************************************************/
-
-
 /*
  * åȤ򥳥ͥ󤫤鳫롣
  * parameters : socket
@@ -1344,15 +1328,17 @@ int releaseSockFromConect(SOCKET *sock)
 		sock->stat |= SOCK_DISCON_RECV | SOCK_DISCON_SEND;
 
 		// ѥåȤ˱
-		while (refIpBuf(sock->recvBuf) != NULL){
+		while (refRecvBuf(sock->recvBuf) != NULL){
+			struct mbuf *mbuf = getDetachRecvBuf(&sock->recvBuf, &sock->lockGate);
+			IP_HEADER *ip;
 			TCP_HEADER *tcp;
-			IP_BUFFER *ipBuf = getRecvBuf((IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
 
-			ASSERT(ipBuf != NULL);
-			sock->acknum += getTcpDataSize(&ipBuf->ip);
-			tcp = (TCP_HEADER*)ipBuf->ip.data;
+			ASSERT(mbuf != NULL);
+			ip = getMbufDataPointer(mbuf);
+			tcp = (TCP_HEADER*)ip->data;
+			sock->acknum += getTcpDataSize(ip);
 			isFin = tcp->flag & TCP_FLG_FIN;
-			freeIpBuf(ipBuf);
+			m_freem(mbuf);
 		}
 		if (isFin && ((sock->stat & SOCK_CNCT_RECV_FIN) == 0)){
 			sendFinAck(sock);
@@ -1367,7 +1353,7 @@ int releaseSockFromConect(SOCKET *sock)
 
 	if (sock->stat & SOCK_LISTEN){
 		delWaitLink(sock);
-		releaseIpBuf(sock->recvBuf);
+		releaseRecvBuf(sock->recvBuf);
 		releaseListenSocket(sock);
 		endListenThread(sock);
 
@@ -1376,13 +1362,20 @@ int releaseSockFromConect(SOCKET *sock)
 
 	if (sock->stat & SOCK_LINK_ON){
 		delWaitLink(sock);
-		releaseIpBuf(sock->recvBuf);
+		releaseRecvBuf(sock->recvBuf);
 	}
 
 	return 0;
 }
 
 
+/**************************************************************************
+ *
+ * åȥӥؿ
+ *
+ **************************************************************************/
+
+
 STATIC int send(SOCKET *sock, void *msg, const size_t i_size, int flags, struct sockaddr *to)
 {
 	enum{
@@ -1477,7 +1470,7 @@ STATIC int send(SOCKET *sock, void *msg, const size_t i_size, int flags, struct
  * FINե饰ͭǥǡͭ			   0 0   0 0     0 0   0 0       1 1   1 1     1 1   1 1       2 2   2 2     2 2   2 2
  * 		 桼ХåեäѤ			   0 0   0 0     1 1   1 1       0 0   0 0     1 1   1 1       0 0   0 0     1 1   1 1
  * 		 ǡ桼Ϥ			   0 0   1 1     0 0   1 1       0 0   1 1     0 0   1 1       0 0   1 1     0 0   1 1
- * 		 ipBufƤ뤫			   0 1   0 1     0 1   0 1       0 1   0 1     0 1   0 1       0 1   0 1     0 1   0 1
+ * 		 ΥǡƤ뤫		   0 1   0 1     0 1   0 1       0 1   0 1     0 1   0 1       0 1   0 1     0 1   0 1
  * 
  * ̡							   0     1       0     1         0     0       0     0         0     1       1     1  
  *      ʺƱѥåȤɬפȤʤ
@@ -1494,8 +1487,6 @@ STATIC int recv(SOCKET *sock, struct msghdr *msg)
 	};
 	char *data;
 	int allSize, tcpDataSize, dataSize, cpSize, iovArrey, iovOffset;
-	IP_BUFFER *ipBuf;
-	IP_HEADER *ip;
 	int rest;
 
 	// ͥγǧ
@@ -1516,25 +1507,28 @@ STATIC int recv(SOCKET *sock, struct msghdr *msg)
 
 	allSize = iovArrey = iovOffset = 0;
 	for (;;){
+		struct mbuf *mbuf;
+		IP_HEADER *ip;
 		TCP_HEADER *tcp;
 		int isFin, isFullUserBuf, isAllDataCopy;
 
 		// 
 		rest = recvData(sock, RECEIVE_TIMEOUT);
-		if (rest < 0){
+		if (rest != NOERR){
 			return rest;
 		}
-		ipBuf = refIpBuf(sock->recvBuf);
-		ip = &ipBuf->ip;
+		mbuf = refRecvBuf(sock->recvBuf);
+		ASSERT(mbuf != NULL);
+		ip = getMbufDataPointer(mbuf);
 		tcp = (TCP_HEADER*)ip->data;
 
 		// 桼Хåե˥ԡ
 		tcpDataSize = getTcpDataSize(ip);
-		dataSize = tcpDataSize - ipBuf->offset;
-		data = ipBuf->ip.data + getTcpSize(ip) - dataSize;
+		dataSize = tcpDataSize - mbuf->mh_offset;
+		data = (char*)tcp + getTcpSize(ip) - dataSize;
 		cpSize = copyIovec(data, dataSize, msg->msg_iov, msg->msg_iovlen, &iovArrey, &iovOffset);
 		allSize += cpSize;
-		ipBuf->offset += cpSize;
+		mbuf->mh_offset += cpSize;
 
 		//FINե饰Ƥ뤫
 		if (tcp->flag & TCP_FLG_FIN){
@@ -1546,7 +1540,7 @@ STATIC int recv(SOCKET *sock, struct msghdr *msg)
 		// 桼ХåեäѤ
 		isFullUserBuf = (iovArrey == msg->msg_iovlen);
 		// ǡ桼Ϥ
-		isAllDataCopy = (ipBuf->offset == tcpDataSize);
+		isAllDataCopy = (mbuf->mh_offset == tcpDataSize);
 
 		ASSERT((isFullUserBuf == 1) || (isAllDataCopy == 1));
 
@@ -1554,12 +1548,12 @@ STATIC int recv(SOCKET *sock, struct msghdr *msg)
 			uint8_t  flag = tcp->flag;
 			
 			// IPХåեγ
-			detachRecvBuf(ipBuf, (IP_BUFFER**)&sock->recvBuf, &sock->lockGate);
-			freeIpBuf(ipBuf);
+			detachRecvBuf(mbuf, &sock->recvBuf, &sock->lockGate);
+			m_freem(mbuf);
 
 			// 
 			sock->acknum += tcpDataSize;
-			if ((flag & (TCP_FLG_PSH | TCP_FLG_FIN)) || (refIpBuf(sock->recvBuf) == NULL)){
+			if ((flag & (TCP_FLG_PSH | TCP_FLG_FIN)) || (refRecvBuf(sock->recvBuf) == NULL)){
 				if (flag & TCP_FLG_FIN){
 					sendFinAck(sock);
 				}
@@ -1568,7 +1562,7 @@ STATIC int recv(SOCKET *sock, struct msghdr *msg)
 				}
 			}
 		}
-		if (isReturn[isFin][isFullUserBuf][isAllDataCopy][refIpBuf(sock->recvBuf) != NULL]){
+		if (isReturn[isFin][isFullUserBuf][isAllDataCopy][refRecvBuf(sock->recvBuf) != NULL]){
 			// λ
 			break;
 		}
@@ -1641,8 +1635,7 @@ STATIC int listen(SOCKET *sock,int fd)
 STATIC int accept(SOCKET *sock,SOCKET **o_newSock,uint32_t *o_srcIp)
 {
 	SOCKET *newSock;
-	IP_BUFFER *ipBuf;
-	TCP_HEADER *tcp;
+	struct mbuf *mbuf;
 	int rest;
 	
 	for (;;){
@@ -1659,52 +1652,39 @@ STATIC int accept(SOCKET *sock,SOCKET **o_newSock,uint32_t *o_srcIp)
 			}
 
 			/* Ԥ */
-			if (sys_sleep(-1) == 0){		/* Time out. */
-				newSock = getFromListenSocket(sock);
-				if (newSock == NULL){
-					return -ETIMEDOUT;
-				}
-				else{
-					break;
-				}
+			sigWait();
+			newSock = getFromListenSocket(sock);
+			if (newSock != NULL){
+				break;
 			}
 		}
+		mbuf = refRecvBuf(newSock->recvBuf);
 
-		ipBuf = refIpBuf(newSock->recvBuf);
 		for (;;){
+			IP_HEADER *ip = getMbufDataPointer(mbuf);
+			TCP_HEADER *tcp = (TCP_HEADER*)ip->data;
+
 			/* γǧ */
-			tcp = (TCP_HEADER*)ipBuf->ip.data;
-			switch (isAck(tcp,newSock->seqnum,newSock->acknum)){
+			switch (isAck(tcp, newSock->seqnum, newSock->acknum)){
 				case -1:
 					/*  */
 				case ACK_LW_SEQ_EQ:
-					detachRecvBuf(ipBuf, (IP_BUFFER**)&newSock->recvBuf, &newSock->lockGate);
-					if (ipBuf->next == NULL){
-						freeIpBuf(ipBuf);
+					detachRecvBuf(mbuf, &newSock->recvBuf, &newSock->lockGate);
+					m_freem(mbuf);
+					mbuf = refRecvBuf(newSock->recvBuf);
+					if (mbuf == NULL){
 						kfree(newSock);
 						goto NEXT;
 					}
-					else{
-						IP_BUFFER *cpIpBuf = ipBuf;
-
-						ipBuf = ipBuf->next;
-						freeIpBuf(cpIpBuf);
-					}
 					break;
 				case ACK_EQ_SEQ_GR:
-					/* ȤΥֹ椬礭ȤϡΥȤ夷ǽ */
-					if (ipBuf->next == NULL){
-						goto NEXT;
-					}
-					else{
-						ipBuf = ipBuf->next;
-					}
+					// ȤΥֹ椬礭ȤϡΥȤ夷ǽ
+					goto NEXT;
 					break;
 				default:
 					/* ׵ */
 					if (tcp->flag & TCP_FLG_RST){
-						detachRecvBuf(ipBuf, (IP_BUFFER**)&newSock->recvBuf, &newSock->lockGate);
-						freeIpBuf(ipBuf);
+						releaseRecvBuf(newSock->recvBuf);
 						kfree(newSock);
 						goto NEXT;
 					}
@@ -1716,8 +1696,8 @@ STATIC int accept(SOCKET *sock,SOCKET **o_newSock,uint32_t *o_srcIp)
 NEXT:	;
 	}
 OUT:
-	detachRecvBuf(ipBuf, (IP_BUFFER**)&newSock->recvBuf, &newSock->lockGate);
-	freeIpBuf(ipBuf);
+	detachRecvBuf(mbuf, &newSock->recvBuf, &newSock->lockGate);
+	m_freem(mbuf);
 
 	newSock->stat |= SOCK_CNCT_ON;
 	newSock->seqnum += 1;
@@ -1755,7 +1735,7 @@ STATIC int poll(SOCKET *sock)
 		}
 	}
 	else{
-		if (refIpBuf(sock->recvBuf) != NULL){
+		if (refRecvBuf(sock->recvBuf) != NULL){
 			return POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM;
 		}
 	}
@@ -1776,12 +1756,10 @@ STATIC int poll(SOCKET *sock)
 static SOCKET_INFO sockInfo = {send, recv, connect, shutdown, listen, accept, poll};
 
 
-/*================================== Х ========================================*/
+//================================== PUBLIC =============================================
 
 
-/*
- * Init TCP.
- */
+// Init TCP.
 int initTcp()
 {
 	registSocket(&sockInfo,IPPROTO_TCP);
