@@ -10,10 +10,11 @@
 #define tcp_h
 
 
-#include<types.h>
+#include<sys/types.h>
+#include<sys/mbuf.h>
 #include<net/net.h>
 #include<net/ip.h>
-#include<mm.h>
+#include<kern/mm.h>
 
 
 #ifndef ASM_FILE
@@ -38,7 +39,7 @@ typedef struct{
 
 
 extern int releaseSockFromConect(SOCKET*);
-extern int receiveTcp(IP_HEADER*);
+extern int receiveTcp(struct mbuf *);
 extern void doTcpService(SOCKET*, IP_HEADER*);
 extern int initTcp();
 
