@@ -165,7 +165,7 @@ volatile sig_atomic_t siginfo_p;
 static void fill(char *, char *);
 static u_short in_cksum(u_short *, int);
 static void check_status(void);
-static void finish(void) __dead2;
+static void finish(void);
 static void pinger(void);
 static char *pr_addr(struct in_addr);
 static void pr_icmph(struct icmp *);
@@ -175,7 +175,7 @@ static void pr_retip(struct ip *);
 static void status(int);
 static void stopit(int);
 static void tvsub(struct timeval *, struct timeval *);
-static void usage(void) __dead2;
+static void usage(void);
 
 int
 main(argc, argv)
