@@ -16,7 +16,7 @@ enum{
 	F_RDLCK,		/* Shared or read lock. */
 	F_UNLCK,		/* Unlock. */
 	F_WRLCK,		/* Exclusive or write lock. */
-	F_FREESP,		/* Free file space */
+	F_FREESP		/* Free file space */
 };
 
 
