@@ -7,7 +7,7 @@
 #define inttypes_h
 
 
-#include<stdint.h>
+#include<sys/inttypes.h>
 
 
 #endif
