@@ -12,7 +12,7 @@
 
 enum{
 	CLOCKS_PER_SEC=100,		/* The number of clock_t units per second. */
-	TM_YEAR_BASE=1900,
+	TM_YEAR_BASE=1900
 };
 
 
