@@ -7,7 +7,7 @@
 #define utime_h
 
 
-#include<share/utime.h>
+#include <sys/utime.h>
 
 
 extern int utime(const char *, struct utimbuf *);
