@@ -3,13 +3,11 @@
  */
 
 
+#include<sys/config.h>
+#include<sys/param.h>
 #include<sys/types.h>
 #include<sys/stat.h>
-
-#include<callgate.h>
-#include<sys/system.h>
 #include<errno.h>
-#include<string.h>
 #include<signal.h>
 #include<stdio.h>
 #include<stropts.h>
