@@ -3,10 +3,7 @@
  */
 
 
-#include<sys/user_config.h>
 #include<sys/types.h>
-#include<callgate.h>
-
 #include<errno.h>
 #include<stdio.h>
 #include<string.h>
