@@ -3,7 +3,7 @@
  */
 
 
-#include<sys/system.h>
+#include<sys/param.h>
 
 
 int getpagesize()
