@@ -3,21 +3,18 @@
  */
 
 
-#include<unistd.h>
 #include<sys/types.h>
-#include<callgate.h>
-
-#include<errno.h>
 #include<fcntl.h>
-#include<sys/stat.h>
 
 
 int mkfifo(const char *pathname, mode_t mode)
 {
 	int fd;
 
-
-	if((fd=creat(pathname,mode|O_FIFO))==-1)return -1;
+	fd = open(pathname, O_WRONLY | O_CREAT | O_EXCL | O_FIFO, mode);
+	if (fd == -1) {
+		return -1;
+	}
 	close(fd);
 
 	return 0;
