@@ -3,16 +3,10 @@
  */
 
 
+#include<sys/config.h>
 #include<sys/types.h>
-#include<sys/stat.h>
-
-#include<callgate.h>
-#include<sys/system.h>
-#include<errno.h>
-#include<string.h>
 #include<signal.h>
 #include<stdio.h>
-#include<stropts.h>
 #include<unistd.h>
 
 
