@@ -28,15 +28,14 @@
 /*	  All Rights Reserved  	*/
 
 
-#include <sys/termios.h>
-#include <unistd.h>
-#include <stropts.h>
 #include <sys/types.h>
+#include <sys/ttycom.h>
+#include <stropts.h>
 
 /*
  * wait until all output on the filedes is drained
  */
 int tcdrain(int fildes)
 {
-	return (ioctl(fildes, TCSBRK, 1));
+	return (ioctl(fildes, TIOCDRAIN, 0));
 }
