@@ -8,10 +8,11 @@
  */
 
 
+#include<sys/config.h>
+#include<sys/param.h>
 #include<unistd.h>
 #include<stdlib.h>
 #include<stdio.h>
-#include<sys/system.h>
 
 
 int main(int a,char **b)
